//  -- LOG --
//
//RPOT 238          21 mai 2020

//DIY CONTROLS:
// - Left Slider Speed
// - Right Pot Leg Lift Height
//
// - A    Walk Mode
// - B    Translate Mode
// - C    SEQ Mode
//        - Init              1
//        - beau              2
//        - lay            
//        - shakeright     
//        - shakeleft      
//        - pattedeb       
//        - shake          
//        - pipi              8
//        - poirier           9
// - D    SEQ Mode
//        - Reverence         1   
//        - Sit               2
//        - sit2           
//        - *Walk           
//        - *RotateRight    
//        - laying         
//        - wakeup         
//        - trick             8
//        - gratte            9
// - 0    Turn on/off the bot


//   Hexapod                    Puppy
// [0]Rr |---| Lr[2]               Lr |---| Rr
//    Rm |   | Lm                     |   |
// [1]Rf |---| Lf[3]               Lf |---| Rf


//Offset: 0            450           -450
//        |               /         \
//      |-|-|         |--/|         |\--|
//      | * |         | * |         | * |
//      | | |         |/  |         |  \|
//      |---|         |---|         |---|

//Body Dimensions:
// cRROffsetX cRROffsetZ

// *-------------------*--------------------*Left
// |                   /\                   |
// |                   |                    |
// |                   X                    |
// |                   |                    |Front
// |                   \/                   |
// |    <----Z---->    0,0                  |
// |                                        |
// *-------------------*--------------------*Right



#define jaw

//mp3
#include "BY8X01-16P.h"
#include "SoftwareSerial.h"
// 0001.mp3 chante chipie
// 0002.wav s'impatiente
// 0003.wav pleure



#include <SendOnlySoftwareSerial.h>  // SSC
#include <EEPROM.h>

//=============================================================================
// Header Files
//=============================================================================

//[CONSTANTS]
#define WALKMODE          0
#define TRANSLATEMODE     1
#define SEQMODE           2

//static const char  *s_asGateNames[] = {"Init", "beau", "lay",
//                "Shake Right", "Shake Left", "patte debout", "Shake L+R", "Piss", "Poirier"};


static byte            bXBeeControlMode;

#define c1DEC   10
#define c2DEC   100
#define c4DEC   10000
#define c6DEC   1000000

#define cRR     0
#define cRF     1
#define cLR     2
#define cLF     3

#define cPwmDiv       991  //old 1059;
#define cPFConst      592  //old 650 ; 900*(1000/cPwmDiv)+cPFConst must always be 1500
// A PWM/deg factor of 10,09 give cPwmDiv = 991 and cPFConst = 592
// For a modified 5645 (to 180 deg travel): cPwmDiv = 1500 and cPFConst = 900.

//--------------------------------------------------------------------
//[Arduino Pin Numbers]
#define cBUSY           3
#define cLLED           5
#define cRLED           6
#define cSSC_OUT        7
#define cTX             8
#define cRX             9

//--------------------------------------------------------------------
//[SERIAL CONNECTIONS]
#define cSSC_BAUD         38400          //SSC32 BAUD rate
#define cMP3_BAUD         9600
SendOnlySoftwareSerial SSCSerial(cSSC_OUT);    // SSC

SoftwareSerial MP3(cRX, cTX); // [RX, TX]
BY8X0116P MP3Serial(MP3, cBUSY); // Library using SoftwareSerial and no busy pin hookup
 

//--------------------------------------------------------------------
//[SSC PIN NUMBERS]
#define cRRCoxaPin      7   //Rear Right leg Hip Horizontal - 0
#define cRRFemurPin     8   //Rear Right leg Hip Vertical - 1

#define cRFCoxaPin      5   //Front Right leg Hip Horizontal - 8 
#define cRFFemurPin     6   //Front Right leg Hip Vertical- 9

#define cLRCoxaPin      3   //Rear Left leg Hip Horizontal- 16
#define cLRFemurPin     4   //Rear Left leg Hip Vertical- 17

#define cLFCoxaPin      1   //Front Left leg Hip Horizontal - 24
#define cLFFemurPin     2   //Front Left leg Hip Vertical - 25

#define cXHeadPin       9
#define cYHeadPin       10

#ifdef jaw
#define cJawPin         11
#endif

//--------------------------------------------------------------------
//[MIN/MAX ANGLES]
#define cRRCoxaMin1     -1500     //Mechanical limits of the Right Rear Leg
#define cRRCoxaMax1     1500
#define cRRFemurMin1    -900
#define cRRFemurMax1    550

#define cRFCoxaMin1     -750      //Mechanical limits of the Right Front Leg
#define cRFCoxaMax1     750
#define cRFFemurMin1    -900
#define cRFFemurMax1    550

#define cLRCoxaMin1     -1500     //Mechanical limits of the Left Rear Leg
#define cLRCoxaMax1     1500
#define cLRFemurMin1    -900
#define cLRFemurMax1    550

#define cLFCoxaMin1     -750      //Mechanical limits of the Left Front Leg
#define cLFCoxaMax1     750
#define cLFFemurMin1    -900
#define cLFFemurMax1    550


#define cXHeadMin1      1100      //HeadXAngle1=min(max(HeadXAngle1,1100),2000);
#define cXHeadMax1      2000
#define cYHeadMin1      955    //HeadYAngle1=min(max(HeadYAngle1,955),1510);
#define cYHeadMax1      1600

#ifdef jaw
#define cYJawMin1      1146    
#define cYJawMax1      1650
#endif
//--------------------------------------------------------------------
//[SERVO OFFSET]
#define cRRCoxaOffset    -60//-60     //Rear Right leg Hip Vertical 
#define cRRFemurOffset   -68     //Rear Right leg Knee        

#define cRFCoxaOffset    -66     //Front Right leg Hip Vertical   
#define cRFFemurOffset   -163//-163    //Front Right leg Knee       

#define cLRCoxaOffset    163//163     //Rear Left leg Hip Vertical     ;OKKKKKK
#define cLRFemurOffset   60      //Rear Left leg Knee

#define cLFCoxaOffset    54//54      //Front Left leg Hip Vertical    ;OKKKKKK
#define cLFFemurOffset   -22//-22     //Front Left leg Knee        

#define cXHeadOffset     65//10
#define cYHeadOffset     20

#ifdef jaw
#define cJawOffset    150
#endif

//-------------------------------------------------------------------
//[BODY DIMENSIONS]
#define cRRCoxaLength     52    //1.14" = 29mm (1.14 * 25.4)
#define cRFCoxaLength     43
#define cLRCoxaLength     52
#define cLFCoxaLength     43

#define cFemurLength      61    //2.25" = 57mm (2.25 * 25.4)
#define cTibiaLength      43

#define cRRCoxaAngle1   0       //Default Coxa setup angle, decimals = 1
#define cRFCoxaAngle1   0       //Default Coxa setup angle, decimals = 1
#define cLRCoxaAngle1   0       //Default Coxa setup angle, decimals = 1
#define cLFCoxaAngle1   0       //Default Coxa setup angle, decimals = 1

//--------------------------------------------------------------------

//[START POSITIONS FEET]
#define cRRInitPosX     93      //Start positions of the Right Rear leg
#define cRRInitPosY     80
#define cRRInitPosZ     0

#define cRFInitPosX     72      //Start positions of the Right Front leg
#define cRFInitPosY     80
#define cRFInitPosZ     0

#define cLRInitPosX     93      //Start positions of the Left Rear leg
#define cLRInitPosY     80
#define cLRInitPosZ     0

#define cLFInitPosX     72      //Start positions of the Left Front leg
#define cLFInitPosY     80
#define cLFInitPosZ     0
//--------------------------------------------------------------------
//[XBEE DEFINE]
#define XBEE_API_PH_SIZE        1      // Changed Packet Header to just 1 byte - Type - we don't use sequence number anyway...
#define XBEE_TRANS_READY        0x01  // Transmitter is ready for requests.
// Optional Word to use in ATDL command
#define XBEE_TRANS_NOTREADY    0x02 // Transmitter is exiting transmitting on the sent DL
// No Extra bytes.
#define XBEE_TRANS_DATA        0x03  // Data Packet from Transmitter to Robot*
// Packet data described below.  Will only be sent when the robot sends
// the remote a XBEE_RECV_REQ_DATA packet and we must return sequence number

#define XBEE_TRANS_SEQ_DATA    0x04           // Ajout 2020

#define XBEE_RECV_DISP_VAL0    0x89
#define XBEE_RECV_DISP_STR     0x85  // Display a string value on line 2 
#define XBEE_RECV_DISP_VAL     0x84  // Display a value on line 2
// If <cbExtra> is  0 then we will display the number contained in <SerialNumber>
// If not zero, then it is a count of bytes in a string to display.

//[Packets sent from Robot to remote]
#define XBEE_RECV_REQ_DATA      0x80  // Request Data Packet*
// No extra bytes: expect to receive XBEE_TRANS_DATA_PACKET
#define XBEE_RECV_REQ_NEW     0x81  // Request Only New data
// No Extra bytes goes into New only mode and we will typically
// wait until Remote says it has new data before asking for data.
// In new mode, the remote may choose to choose a threshold of how big a change
// needs to be before sending the XBEE_TRANS_NEW value.


//[XBEE_TRANS_DATA] - has 8 extra bytes
//  0 - Buttons High
//  1 - Buttons Low
//  2 - Right Joystick L/R
//  3 - Right Joystick U/D
//  4 - Left Joystick L/R
//  5 - Left Joystick U/D
//  6 - Right Slider
//  7 - Left Slider


// Main data packet
typedef union {
  byte    ab[12];
  struct  {
    uint16_t  wButtons;     // the 16 buttons
    byte  bRJoyLR;          // right joystick X (LR)
    byte  bRJoyUD;          //        ..Y (UD)
    byte  bLJoyLR;          // Left Joystick  X (LR)
    byte  bLJoyUD;          //          Y (UD)
    // Added values since first version
    byte    bRPot;          // top Pot on right joystick
    byte    bLPot;          // ... Left joystick
    byte  bRSlider;         // Right Slider
    byte  bLSlider;         // Left Slider
    byte  bMSlider;         // Middle Slider
    byte    bButtons2;      // Extra buttons on new DIY...
  }
  s;
}
DIYPACKET;

typedef DIYPACKET *PDIYPACKET;

typedef unsigned long ulong;


typedef struct _diypacketOrig {
  uint16_t  wButtons;         // the 16 buttons
  uint8_t bRJoyLR;            // right joystick X (LR)
  uint8_t bRJoyUD;            //        ..Y (UD)
  uint8_t bLJoyLR;            // Left Joystick  X (LR)
  uint8_t bLJoyUD;            //          Y (UD)
  uint8_t bRPot;              // Right Slider
  uint8_t bLPot;              // Left Slider
}
DIYPACKETORIG;


// Now define some static state stuff
typedef struct _diystate
{
  // state information
  boolean   fTransReadyRecvd;
  boolean   fPacketValid;
  boolean   fPacketTimeOut;
  boolean   fPacketForced;
  boolean   fNewPacket;
  boolean   fDataPacketsReceived;
  uint8_t   cbPacketSize;    // what was the size of the last data packet received.
  // More internal to function...

  boolean   fReqDataPacketSent;
  boolean   fReqDataForced;
  boolean   fSendOnlyNewMode;
  byte      bPacketNum;
  uint8_t   bTxStatusLast;
  byte      bAPIPacket[33];       // Api packet

  // Other information, could make static to file...
  ulong ulLastPacket;

} DIYSTATE;

DIYSTATE g_diystate;



// Some externals, probably should move somewhere...

DIYPACKET  g_diyp;
DIYPACKET  g_diypPrev;



//--------------------------------------------------------------------
//[TABLES]
//ArcCosinus Table
//Table build in to 3 part to get higher accuracy near cos = 1.
//The biggest error is near cos = 1 and has a biggest value of 3*0.012098rad = 0.521 deg.
//-    Cos 0 to 0.9 is done by steps of 0.0079 rad. [1/127]
//-    Cos 0.9 to 0.99 is done by steps of 0.0008 rad [0.1/127]
//-    Cos 0.99 to 1 is done by step of 0.0002 rad [0.01/64]
//Since the tables are overlapping the full range of 127+127+64 is not necessary. Total bytes: 277

static const byte GetACos[] PROGMEM = {
  255, 254, 252, 251, 250, 249, 247, 246, 245, 243, 242, 241, 240, 238, 237, 236, 234, 233, 232, 231, 229, 228, 227, 225,
  224, 223, 221, 220, 219, 217, 216, 215, 214, 212, 211, 210, 208, 207, 206, 204, 203, 201, 200, 199, 197, 196, 195, 193,
  192, 190, 189, 188, 186, 185, 183, 182, 181, 179, 178, 176, 175, 173, 172, 170, 169, 167, 166, 164, 163, 161, 160, 158,
  157, 155, 154, 152, 150, 149, 147, 146, 144, 142, 141, 139, 137, 135, 134, 132, 130, 128, 127, 125, 123, 121, 119, 117,
  115, 113, 111, 109, 107, 105, 103, 101, 98, 96, 94, 92, 89, 87, 84, 81, 79, 76, 73, 73, 73, 72, 72, 72, 71, 71, 71, 70, 70,
  70, 70, 69, 69, 69, 68, 68, 68, 67, 67, 67, 66, 66, 66, 65, 65, 65, 64, 64, 64, 63, 63, 63, 62, 62, 62, 61, 61, 61, 60, 60, 59,
  59, 59, 58, 58, 58, 57, 57, 57, 56, 56, 55, 55, 55, 54, 54, 53, 53, 53, 52, 52, 51, 51, 51, 50, 50, 49, 49, 48, 48, 47, 47, 47,
  46, 46, 45, 45, 44, 44, 43, 43, 42, 42, 41, 41, 40, 40, 39, 39, 38, 37, 37, 36, 36, 35, 34, 34, 33, 33, 32, 31, 31, 30, 29, 28,
  28, 27, 26, 25, 24, 23, 23, 23, 23, 22, 22, 22, 22, 21, 21, 21, 21, 20, 20, 20, 19, 19, 19, 19, 18, 18, 18, 17, 17, 17, 17, 16,
  16, 16, 15, 15, 15, 14, 14, 13, 13, 13, 12, 12, 11, 11, 10, 10, 9, 9, 8, 7, 6, 6, 5, 3, 0
};//


//Build tables for Leg configuration like I/O and MIN/imax values to easy access values using a FOR loop
//Constants are still defined as single values in the cfg file to make it easy to read/configure

//SSC Pin numbers
static const byte cCoxaPin[] PROGMEM  = {cRRCoxaPin, cRFCoxaPin, cLRCoxaPin, cLFCoxaPin};
static const byte cFemurPin[] PROGMEM = {cRRFemurPin, cRFFemurPin, cLRFemurPin, cLFFemurPin};

static const byte cHeadPin[] PROGMEM  = {cXHeadPin, cYHeadPin, cJawPin};           // JAW ICI

//Min / imax values
static const short cCoxaMin1[] PROGMEM  = {cRRCoxaMin1,  cRFCoxaMin1,  cLRCoxaMin1,  cLFCoxaMin1};
static const short cCoxaMax1[] PROGMEM  = {cRRCoxaMax1,  cRFCoxaMax1,  cLRCoxaMax1,  cLFCoxaMax1};
static const short cFemurMin1[] PROGMEM = {cRRFemurMin1, cRFFemurMin1, cLRFemurMin1, cLFFemurMin1};
static const short cFemurMax1[] PROGMEM = {cRRFemurMax1, cRFFemurMax1, cLRFemurMax1, cLFFemurMax1};

static const short cHeadMin1[] PROGMEM = {cXHeadMin1, cYHeadMin1,cYJawMin1};
static const short cHeadMax1[] PROGMEM = {cXHeadMax1, cYHeadMax1, cYJawMax1}; 

//Servo Offsets
static const short cCoxaOffset[] PROGMEM  = {cRRCoxaOffset, cRFCoxaOffset, cLRCoxaOffset, cLFCoxaOffset};
static const short cFemurOffset[] PROGMEM = {cRRFemurOffset, cRFFemurOffset, cLRFemurOffset, cLFFemurOffset};

static const short cHeadOffset[] PROGMEM = {cXHeadOffset, cYHeadOffset, cJawOffset};

//Start positions for the leg
static const short cInitPosX[] PROGMEM = {cRRInitPosX, cRFInitPosX, cLRInitPosX, cLFInitPosX};
static const short cInitPosY[] PROGMEM = {cRRInitPosY, cRFInitPosY, cLRInitPosY, cLFInitPosY};
static const short cInitPosZ[] PROGMEM = {cRRInitPosZ, cRFInitPosZ, cLRInitPosZ, cLFInitPosZ};

static const short cCoxaLength[] PROGMEM  = {cRRCoxaLength, cRFCoxaLength, cLRCoxaLength, cLFCoxaLength};

static const byte cEyesPin[] PROGMEM  = {cLLED, cRLED};
//--------------------------------------------------------------------
//[REMOTE]
#define cTravelDeadZone       4  //The deadzone for the analog input from the remote
//====================================================================
//[ANGLES]
short     CoxaAngle1[4];         //Actual Angle of the horizontal hip, decimals = 1
short     FemurAngle1[4];        //Actual Angle of the vertical hip, decimals = 1

short     HeadXAngle1;
short     HeadYAngle1;

#ifdef jaw
short     JawAngle1;
#endif

          // --  pour dT si inactif
short   prev_CoxaAngle[4];
short   prev_FemurAngle[4];

short   prev_HeadXAngle;
short   prev_HeadYAngle;
//--------------------------------------------------------------------
//[POSITIONS SINGLE LEG CONTROL]
short      LegPosX[4];           //Actual X Posion of the Leg
short      LegPosY[4];           //Actual Y Posion of the Leg
short      LegPosZ[4];           //Actual Z Posion of the Leg

//--------------------------------------------------------------------
//[VARIABLES]
byte            Index;           //Index universal used
byte            LegIndex;        //Index used for leg Index Number

//GetSinCos / ArcCos
short           AngleDeg1;       //Input Angle in degrees, decimals = 1
short           sin4;            //Output Sinus of the given Angle, decimals = 4
short           cos4;            //Output Cosinus of the given Angle, decimals = 4
short           AngleRad4;       //Output Angle in radials, decimals = 4

//GetAtan2
short           AtanX;            //Input X
short           AtanY;            //Input Y
short           Atan4;            //ArcTan2 output
short           XYhyp2;           //Output presenting Hypotenuse of X and Y

//Body position
long            BodyPosX;         //Global Input for the position of the body
long            BodyPosY;
long            BodyPosZ;

// Prev Body Position             -> test pour eviter saccade
long            PrevBodyPosX;         
long            PrevBodyPosY;
long            PrevBodyPosZ;


//Leg Inverse Kinematics
long            IKFeetPosX;        //Input position of the Feet X
long            IKFeetPosY;        //Input position of the Feet Y
long            IKFeetPosZ;        //Input Position of the Feet Z

long BPosZ;
//--------------------------------------------------------------------
//[TIMING]
unsigned long   lTimerStart;       //Start time of the calculation cycles
unsigned long   lTimerEnd;         //End time of the calculation cycles
byte            CycleTime;         //Total Cycle time

word            SSCTime;           //Time for servo updates
word            PrevSSCTime;       //Previous time for the servo updates

byte            InputTimeDelay;    //Delay that depends on the input to get the "sneaking" effect
//--------------------------------------------------------------------
//[GLOABAL]
boolean          g_fPuppyIKOn;      //Switch to turn on Phoenix
boolean          g_fPrev_PupIKOn;   //Previous loop state

//--------------------------------------------------------------------
//[gait]
byte             GaitType;           //Gait type
short            NomGaitSpeed;       //Nominal speed of the gait

short            LegLiftHeight;      //Current Travel height
long             TravelLengthX;      //Current Travel length X
long             TravelLengthZ;      //Current Travel length Z

short            TLDivFactor;        //Number of steps that a leg is on the floor while walking
short            NrLiftedPos;        //Number of positions that a single leg is lifted [1-3]
boolean          HalfLiftHeigth;     //If TRUE the outer positions of the ligted legs will be half height

byte             LiftDivFactor;      //ADD
boolean          GaitInMotion;       //Temp to check if the gait is in motion
byte             StepsInGait;        //Number of steps in gait
boolean          LastLeg;            //TRUE when the current leg is the last leg of the sequence
byte             GaitStep;           //Actual Gait step

byte             GaitLegNr[4];       //Init position of the leg

long             GaitPosX[4];        //Array containing Relative X position corresponding to the Gait
long             GaitPosY[4];        //Array containing Relative Y position corresponding to the Gait
long             GaitPosZ[4];        //Array containing Relative Z position corresponding to the Gait

boolean          fWalking;           //  True if the robot are walking
boolean          fContinueWalking;   // should we continue to walk?

//[FK Control]

byte             SelectMode;

//--------------------------------------------------------------------

char             OutputString[15] = {'\0'};

// [LIPO]
bool              g_fLowVoltageShutdown;
word              Voltage;

word              g_RawAnalog;                 // Keep 8 raw values
byte              iRaw;
byte              iCntLow;

unsigned long     ulLastSend;
// -----

// [Seq Data]
byte            abPin[12];
bool            SeqOk;

//[EEPROM]
uint16_t         Ptr[10];              //Pointer EEPROM
int              Addr = 0;            //Address EEPROM

//[Playing step]
//byte             currentStep;
//byte             SelectedSeq;         // Seq selectionnée
//byte             nTimes;              //repete x fois une seq

//byte             maxStep;             // Step max de la sequence
//byte             GuardStep;

boolean             SoundOn;
unsigned long       IddleTime;
boolean                bPlay;

int fadeValue;






#define XBeeOutputString(pString)   {SendXBeePacket(XBEE_RECV_DISP_STR, strlen(pString), (byte*)pString);}
//=============================================================================
// Function prototypes
//=============================================================================
void GaitSelect(void);
void GaitSeq(void);
void ServoDriverStart(void);
void ServoDriverCommit(void);
void CheckAngles();
void LegIK (short IKFeetPosX, short IKFeetPosY, short IKFeetPosZ, byte LegIKLegNr);
void Gait (byte GaitCurrentLegNr);
short GetATan2 (short AtanX, short AtanY);

void XBeeControlInput(void);
void SendXBeePacket(byte bPHType, byte cbExtra, byte *pbExtra);
void XBeeOutputVal(word w);
boolean ReceiveXBeePacket(PDIYPACKET pdiyp);
void ClearXBeeInputBuffer(void);
void WriteToXBee(byte *pb, byte cb);

// The defined controller must provide the following
void InitController(void);

// maybe should do this different
void ServoDriverStart(void);
void ServoDriverCommit(void);

short SmoothControl (short CtrlMoveInp, short CtrlMoveOut, byte CtrlDivider);
//void Guard(void);
void Led(int val);
void WriteOutputs(byte Index, int fadeV);
short UnMap(byte bPin);
void OutputSrevoJaw(void);

//--------------------------------------------------------------------------
// SETUP: the main arduino setup function.
//--------------------------------------------------------------------------
void setup() {
  
  pinMode(A7,INPUT);        // BAttery
  
  pinMode(cLLED,OUTPUT);
  pinMode(cRLED,OUTPUT);
  
  pinMode(cTX,OUTPUT);      // Forcing parfois ne prend pas
  
  MP3.begin(cMP3_BAUD);
  MP3Serial.init();
  MP3Serial.setVolume(28); 
  
  SSCSerial.begin(cSSC_BAUD);

  SoundOn = true;
  
   //Tars Init Positions
  for (LegIndex = 0; LegIndex <= 3; LegIndex++ )
  {
    LegPosX[LegIndex] = (short)pgm_read_word(&cInitPosX[LegIndex]);    //Set start positions for each leg
    LegPosY[LegIndex] = (short)pgm_read_word(&cInitPosY[LegIndex]);
    LegPosZ[LegIndex] = (short)pgm_read_word(&cInitPosZ[LegIndex]);
  }

  //Body Positions
  BodyPosX = 0;
  BodyPosY = 0;
  BodyPosZ = 0;

  //Gait
  GaitType = 0;
  LegLiftHeight = 50;
  GaitStep =  1;
  GaitSelect();

  //Initialize Controller
  InitController();

  //SSC
  SSCTime = 600;
  g_fPuppyIKOn = 0;


WriteOutputs(0,255);// a tester si  avec 0 ok
WriteOutputs(1,255);
delay(100);
WriteOutputs(0,0);
WriteOutputs(1,0);

IddleTime=millis();// temps d'attente si> 6000 play sound
  

}


//=============================================================================
// Loop: the main arduino main Loop function
//=============================================================================


void loop(void)
{
  //Start time
  lTimerStart = millis();
 BPosZ=0;
  
  //Read input
  CheckVoltage();        // check our voltages...
  //if (!g_fLowVoltageShutdown)
        XBeeControlInput();
  
  
  switch (SelectMode){
    case 0:
    //Gait
    GaitSeq();

    //Do IK for all Right legs
    for (LegIndex = 0; LegIndex <= 1; LegIndex++) {
      if (LegIndex == 1) {
        LegIK (LegPosX[LegIndex] - BodyPosX - (GaitPosX[LegIndex] ),
               LegPosY[LegIndex] - 44 + GaitPosY[LegIndex] ,
               LegPosZ[LegIndex] + BodyPosZ + GaitPosZ[LegIndex] +BPosZ, LegIndex);
      } else {
        LegIK (LegPosX[LegIndex] - BodyPosX  - (GaitPosX[LegIndex] ),
               LegPosY[LegIndex] + BodyPosY + GaitPosY[LegIndex],
               LegPosZ[LegIndex] + BodyPosZ + GaitPosZ[LegIndex] +BPosZ, LegIndex);
      }

    }

    //Do IK for all Left legs
    for (LegIndex = 2; LegIndex <= 3; LegIndex++) {
      if (LegIndex == 3) {
        LegIK (LegPosX[LegIndex] + BodyPosX + GaitPosX[LegIndex] ,
               LegPosY[LegIndex] - 44 + GaitPosY[LegIndex] ,
               LegPosZ[LegIndex] + BodyPosZ + GaitPosZ[LegIndex] +BPosZ, LegIndex);
      } else {
        LegIK (LegPosX[LegIndex] + BodyPosX + GaitPosX[LegIndex] ,
               LegPosY[LegIndex] + BodyPosY + GaitPosY[LegIndex] ,
               LegPosZ[LegIndex] + BodyPosZ + GaitPosZ[LegIndex] -BPosZ, LegIndex);

      }
   
    }
    
    //Check mechanical limits
    CheckAngles();


    //Drive Servos
    if (g_fPuppyIKOn) {


      //Set SSC time
      if ((abs(TravelLengthX) > cTravelDeadZone) || (abs(TravelLengthZ) > cTravelDeadZone) || (abs(HeadXAngle1) > cTravelDeadZone) || (abs(HeadYAngle1) > cTravelDeadZone)) {
        SSCTime = NomGaitSpeed + (InputTimeDelay * 2); 
        
        if (bXBeeControlMode==TRANSLATEMODE)
                SSCTime = SSCTime + 50;
                 
      } else
        SSCTime = 200; //old 200


      
       //pour eviter saccades
       StartUpdateServos();

      // Sync BAP with SSC while walking to ensure the prev is completed before sending the next one

      fContinueWalking = false;

      // Finding any incident of GaitPos/Rot <>0:
      for (LegIndex = 0; LegIndex <= 3; LegIndex++) {
        if ( (GaitPosX[LegIndex] > 2) || (GaitPosX[LegIndex] < -2)
             || (GaitPosY[LegIndex] > 2) || (GaitPosY[LegIndex] < -2)
             || (GaitPosZ[LegIndex] > 2) || (GaitPosZ[LegIndex] < -2) ) {
             
          fContinueWalking = true;
          break;
        }
      }
      if ((abs(HeadXAngle1) > cTravelDeadZone) || (abs(HeadYAngle1) > cTravelDeadZone))
          fContinueWalking = true;
          
      //pour eviter saccades
      /*if ( (g_diyp.s.bRJoyLR != g_diypPrev.s.bRJoyLR) || (g_diyp.s.bLJoyLR != g_diypPrev.s.bLJoyLR) || (g_diyp.s.bRJoyUD != g_diypPrev.s.bRJoyUD) ||
        (g_diyp.s.bLJoyUD != g_diypPrev.s.bLJoyUD) || (g_diyp.s.bLPot != g_diypPrev.s.bLPot) ||  (g_diyp.s.bRPot != g_diypPrev.s.bRPot) ||
        (fContinueWalking) || (BodyPosX != PrevBodyPosX) || (BodyPosY != PrevBodyPosY) || (BodyPosZ != PrevBodyPosZ) ){
          StartUpdateServos();
          ServoDriverCommit();
       }*/
        
      if (fWalking || fContinueWalking) {
        word  wDelayTime;
        fWalking = fContinueWalking;

        //Get endtime and calculate wait time
        lTimerEnd = millis();
        if (lTimerEnd > lTimerStart)
          CycleTime = lTimerEnd - lTimerStart;
        else
          CycleTime = 0xffffffffL - lTimerEnd + lTimerStart + 1;

        // if it is less, use the last cycle time...
        //Wait for previous commands to be completed while walking
        wDelayTime = (min(max ((PrevSSCTime - CycleTime), 1), NomGaitSpeed));
        
        Led(255);          
        delay (wDelayTime);
        Led(128);
       
     }else{
      delay(SSCTime);
     }
    
       //pour eviter saccades
      ServoDriverCommit();
      
       
      PrevBodyPosX=BodyPosX;
      PrevBodyPosY=BodyPosY;
      PrevBodyPosZ=BodyPosZ;

      
      if ((g_diyp.s.bLPot==50) &&  (MP3Serial.isBusy()==false) && (bPlay==false)){
              MP3Serial.playFileIndex(2);
              bPlay=true;
        }

         
     if ((millis()> IddleTime+25000) &&  (MP3Serial.isBusy()==false)) {
            MP3Serial.playFileIndex(3);
            IddleTime=millis();
     }
  
    } else {
      //Turn the bot off
      if (g_fPrev_PupIKOn ){  
        SSCTime = 600;
        StartUpdateServos();
        ServoDriverCommit();
        delay(SSCTime);
      }

      delay(20);
    }

   
    
  break;
  case 1:
        if (SeqOk){
        StartUpdateServos();
        ServoDriverCommit();
        
        if ((abPin[11]==50) &&  (MP3Serial.isBusy()==false) && (bPlay==false)){
              MP3Serial.playFileIndex(1);
              bPlay=true;
        }
        
        if ((millis()> IddleTime+25000) &&  (MP3Serial.isBusy()==false)) {
            MP3Serial.playFileIndex(3);
            IddleTime=millis();
        }
           
        delay(SSCTime);
        SeqOk=false;
        }

       
  break;
  
  default:
  break;
  }

   
   
  //Store previous g_fPuppyIKOn State
  if (g_fPuppyIKOn)
    g_fPrev_PupIKOn = 1;
    else
    g_fPrev_PupIKOn = 0;
  
}

//--------------------------------------------------------------------
//[StartUpdateServos]
void StartUpdateServos()
{
  byte    LegIndex;

  // First call off to the init...
  for (LegIndex = 0; LegIndex <= 3; LegIndex++) {
      OutputServoInfoForLeg(LegIndex, CoxaAngle1[LegIndex], FemurAngle1[LegIndex]);
  }

  OutputServoHead();
  
  #ifdef jaw
  OutputServoJaw();
  #endif
}

//--------------------------------------------------------------------
//[OUTPUT SERVO HEAD]

void OutputServoHead(void)
{
  word    wHeadXSSCV;
  word    wHeadYSSCV;
    
    wHeadXSSCV = ((long)(HeadXAngle1*10 + 900)) * 1000 / cPwmDiv + cPFConst + (short)pgm_read_word(&cHeadOffset[0]);
    wHeadYSSCV = ((long)((long)(HeadYAngle1*10 + 900)) * 1000 / cPwmDiv + cPFConst + (short)pgm_read_word(&cHeadOffset[1]));
  
    //ex 89-128 not 1510 1520
    wHeadXSSCV=min(max(wHeadXSSCV,(short)pgm_read_word(&cHeadMin1[0])),(short)pgm_read_word(&cHeadMax1[0]));
    wHeadYSSCV=min(max(wHeadYSSCV,(short)pgm_read_word(&cHeadMin1[1])),(short)pgm_read_word(&cHeadMax1[1]));
  
    SSCSerial.print("#");
    SSCSerial.print(pgm_read_byte(&cHeadPin[0]), DEC);
    SSCSerial.print("P");
    SSCSerial.print(wHeadXSSCV, DEC);
    
    SSCSerial.print("#");
    SSCSerial.print(pgm_read_byte(&cHeadPin[1]), DEC);
    SSCSerial.print("P");
    SSCSerial.print(wHeadYSSCV, DEC);
   
}

//--------------------------------------------------------------------
//[OUTPUT SERVO HEAD]
// separé pour etre appellée quand son
void OutputServoJaw(void)
{
  word    wJawSSCV;
  wJawSSCV= ((long)(JawAngle1*10 + 900)) * 1000 / cPwmDiv + cPFConst + (short)pgm_read_word(&cHeadOffset[2]);
  wJawSSCV=min(max(wJawSSCV,(short)pgm_read_word(&cHeadMin1[2])),(short)pgm_read_word(&cHeadMax1[2]));

    SSCSerial.print("#");
    SSCSerial.print(pgm_read_byte(&cHeadPin[2]), DEC);
    SSCSerial.print("P");
    SSCSerial.print(wJawSSCV, DEC);

}


//--------------------------------------------------------------------
//[
]
void GaitSelect(void)
{
  //Gait selector
  //;shifting from (LR-RF)(RR-LF)
  if (GaitType == 0) {
    GaitLegNr[cLR] = 1;
    GaitLegNr[cRF] = 2;
    GaitLegNr[cRR] = 5;
    GaitLegNr[cLF] = 7;

    NrLiftedPos = 2;
    HalfLiftHeigth = 0;//0;
    TLDivFactor = 6;
    StepsInGait = 9;
    NomGaitSpeed = 60;//100;
  }    
    
  if (GaitType ==1 ) {
    GaitLegNr[cLR] = 3;//3;    
    GaitLegNr[cRF] = 3;//3;
    GaitLegNr[cRR] = 1;
    GaitLegNr[cLF] = 1;

    NrLiftedPos = 1 ;
    HalfLiftHeigth = 0;
    TLDivFactor = 2;   
    StepsInGait = 10;//4;     
    NomGaitSpeed = 100;
 }
 
 if (GaitType ==2 ) {
    GaitLegNr[cLR] = 1;//3;    
    GaitLegNr[cRF] = 1;//3;
    GaitLegNr[cRR] = 3;
    GaitLegNr[cLF] = 3;

    NrLiftedPos = 1 ;
    HalfLiftHeigth = 0;//0;
    TLDivFactor = 2;   
    StepsInGait = 10;//4;     
    NomGaitSpeed = 100;
  }
}

//--------------------------------------------------------------------
//[GAIT Sequence]
void GaitSeq(void)
{
  
  
  //Calculate Gait sequence
  LastLeg = 0;

  for (LegIndex = 0; LegIndex <= 3; LegIndex++) { // for all legs
    if (LegIndex == 3) // last leg
      LastLeg = 1 ;

    Gait(LegIndex);
  }    // next leg
}


//--------------------------------------------------------------------
//[GAIT]
void Gait (byte GaitCurrentLegNr)
{

  //Check if the Gait is in motion
  GaitInMotion = ((abs(TravelLengthX) > cTravelDeadZone) || (abs(TravelLengthZ) > cTravelDeadZone));

  //Clear values under the cTravelDeadZone
  if (GaitInMotion == 0) {
    TravelLengthX = 0;
    TravelLengthZ = 0;
  }

  //Leg middle up position
  //Gait in motion                                                          Gait NOT in motion, return to home position
  if ( (GaitInMotion && (NrLiftedPos == 1 || NrLiftedPos == 3) && GaitStep == GaitLegNr[GaitCurrentLegNr])
       || ( !GaitInMotion && GaitStep == GaitLegNr[GaitCurrentLegNr] &&
            ((abs(GaitPosX[GaitCurrentLegNr]) > 2) || (abs(GaitPosZ[GaitCurrentLegNr]) > 2) ))) {      //Up
    GaitPosX[GaitCurrentLegNr] = 0;
    GaitPosY[GaitCurrentLegNr] = -LegLiftHeight;
    GaitPosZ[GaitCurrentLegNr] = 0;

  }  else {
    //Optional Half heigth Rear
    if ( ((NrLiftedPos == 2 && GaitStep == GaitLegNr[GaitCurrentLegNr]) //+2
          || (NrLiftedPos == 3 && (GaitStep == (GaitLegNr[GaitCurrentLegNr] - 1) || GaitStep == (GaitLegNr[GaitCurrentLegNr] + (StepsInGait - 1)) )))
         && GaitInMotion) {
      GaitPosX[GaitCurrentLegNr] = -TravelLengthX / 2 ;
      GaitPosY[GaitCurrentLegNr] = -LegLiftHeight / (HalfLiftHeigth + 1);
      GaitPosZ[GaitCurrentLegNr] = -TravelLengthZ / 2;
    } else {
      //Optional half heigth front
      if (  (NrLiftedPos >= 2) && (
              (GaitStep == (GaitLegNr[GaitCurrentLegNr] + 1) || (GaitStep == (GaitLegNr[GaitCurrentLegNr] - (StepsInGait - 1))) )) &&
            GaitInMotion)  {
        GaitPosX[GaitCurrentLegNr] = TravelLengthX / 2;
        GaitPosY[GaitCurrentLegNr] = -LegLiftHeight / (HalfLiftHeigth + 1);
        //GaitPosZ[GaitCurrentLegNr] = TravelLengthZ / 2;                                 
        
        if ( (TravelLengthZ<-cTravelDeadZone) && (TravelLengthX >= -cTravelDeadZone && TravelLengthX <= cTravelDeadZone) ){
            if (GaitCurrentLegNr==0 || GaitCurrentLegNr==2)
                GaitPosZ[GaitCurrentLegNr] = TravelLengthZ*12 ;//10
            else
                GaitPosZ[GaitCurrentLegNr] = -TravelLengthZ*8;//6
        
        }else
            GaitPosZ[GaitCurrentLegNr] = TravelLengthZ / 2;
          
      } else {
        //Leg front down position
        if ((GaitStep == (GaitLegNr[GaitCurrentLegNr] + NrLiftedPos)
             || GaitStep == (GaitLegNr[GaitCurrentLegNr] - (StepsInGait - NrLiftedPos))) && (GaitPosY[GaitCurrentLegNr] < 0) ) {
          GaitPosX[GaitCurrentLegNr] = TravelLengthX / 2;
          GaitPosZ[GaitCurrentLegNr] = TravelLengthZ / 2;
          GaitPosY[GaitCurrentLegNr] = 0;    //Only move leg down at once if terrain adaption is turned off

        }
        //Move body forward
        else {
          GaitPosX[GaitCurrentLegNr] -= ((long)TravelLengthX / (long)TLDivFactor);
          GaitPosY[GaitCurrentLegNr] = 0  ;
          GaitPosZ[GaitCurrentLegNr] -=  ((long)TravelLengthZ / (long)TLDivFactor);

        }
      }
    }
  }
  //Advance to the next step
  if (LastLeg)  {  //The last leg in this step
    GaitStep = GaitStep + 1;
    if (GaitStep > StepsInGait)
      GaitStep = 1;
  }
}

//--------------------------------------------------------------------
//(GETARCCOS) Get the sinus and cosinus from the angle +/- multiple circles
//cos4        - Input Cosinus
//AngleRad4     - Output Angle in AngleRad4
long GetArcCos(short cos4)
{
  boolean NegativeValue/*:1*/;    //If the the value is Negative
  //Check for negative value
  if (cos4 < 0)
  {
    cos4 = -cos4;
    NegativeValue = 1;
  }
  else
    NegativeValue = 0;

  //Limit cos4 to his maximal value
  cos4 = min(cos4, c4DEC);

  if ((cos4 >= 0) && (cos4 < 9000))
  {
    AngleRad4 = (byte)pgm_read_byte(&GetACos[cos4 / 79]);
    AngleRad4 = ((long)AngleRad4 * 616) / c1DEC;        //616=acos resolution (pi/2/255) ;
  }
  else if ((cos4 >= 9000) && (cos4 < 9900))
  {
    AngleRad4 = (byte)pgm_read_byte(&GetACos[(cos4 - 9000) / 8 + 114]);
    AngleRad4 = (long)((long)AngleRad4 * 616) / c1DEC;         //616=acos resolution (pi/2/255)
  }
  else if ((cos4 >= 9900) && (cos4 <= 10000))
  {
    AngleRad4 = (byte)pgm_read_byte(&GetACos[(cos4 - 9900) / 2 + 227]);
    AngleRad4 = (long)((long)AngleRad4 * 616) / c1DEC;         //616=acos resolution (pi/2/255)
  }

  //Add negative sign
  if (NegativeValue)
    AngleRad4 = 31416 - AngleRad4;

  return AngleRad4;
}

//--------------------------------------------------------------------
//(ISQRT32) - Sqrt
unsigned long isqrt32 (unsigned long n) //
{
  unsigned long root;
  unsigned long remainder;
  unsigned long  place;

  root = 0;
  remainder = n;
  place = 0x40000000; // OR place = 0x4000; OR place = 0x40; - respectively

  while (place > remainder)
    place = place >> 2;
  while (place)
  {
    if (remainder >= root + place)
    {
      remainder = remainder - root - place;
      root = root + (place << 1);
    }
    root = root >> 1;
    place = place >> 2;
  }
  return root;
}


//--------------------------------------------------------------------
//(GETATAN2) Simplyfied ArcTan2 function based on fixed point ArcCos
//ArcTanX         - Input X
//ArcTanY         - Input Y
//ArcTan4          - Output ARCTAN2(X/Y)
//XYhyp2            - Output presenting Hypotenuse of X and Y
short GetATan2 (short AtanX, short AtanY)
{
  XYhyp2 = isqrt32(((long)AtanX * AtanX * c4DEC) + ((long)AtanY * AtanY * c4DEC));
  GetArcCos (((long)AtanX * (long)c6DEC) / (long) XYhyp2);

  if (AtanY < 0)                // removed overhead... Atan4 = AngleRad4 * (AtanY/abs(AtanY));
    Atan4 = -AngleRad4;
  else
    Atan4 = AngleRad4;

  return Atan4;
}

//--------------------------------------------------------------------
//[LEG INVERSE KINEMATICS] Calculates the angles of the coxa, femur and tibia for the given position of the feet
//IKFeetPosX            - Input position of the Feet X
//IKFeetPosY            - Input position of the Feet Y
//IKFeetPosZ            - Input Position of the Feet Z
//FemurAngle1           - Output Angle of Femur in degrees
//CoxaAngle1            - Output Angle of Coxa in degrees
//--------------------------------------------------------------------
void LegIK (short IKFeetPosX, short IKFeetPosY, short IKFeetPosZ, byte LegIKLegNr)
{
  unsigned long    IKSW2;            //Length between Shoulder and Wrist, decimals = 2
  unsigned long    IKA14;            //Angle of the line S>W with respect to the ground in radians, decimals = 4
  unsigned long    IKA24;            //Angle of the line S>W with respect to the femur in radians, decimals = 4
  short            IKFeetPosXZ;    //Diagonal direction from Input X and Z
  long             Temp1;
  long             Temp2;
  long             T3;

  //Calculate IKCoxaAngle and IKFeetPosXZ
  GetATan2 (IKFeetPosX, IKFeetPosZ);
  CoxaAngle1[LegIKLegNr] = (((long)Atan4 * 180) / 3141);

  //Length between the Coxa and tars [foot]
  IKFeetPosXZ = XYhyp2 / c2DEC;

  //Using GetAtan2 for solving IKA1 and IKSW
  //IKA14 - Angle between SW line and the ground in radians
  IKA14 = GetATan2 (IKFeetPosY, IKFeetPosXZ - (byte)pgm_read_byte(&cCoxaLength[LegIKLegNr]));//(byte)pgm_read_byte

  //IKSW2 - Length between femur axis and tars
  IKSW2 = XYhyp2;

  //IKA2 - Angle of the line S>W with respect to the femur in radians
  Temp1 = ((((long)cFemurLength * cFemurLength) - ((long)cTibiaLength * cTibiaLength)) * c4DEC + ((long)IKSW2 * IKSW2));
  Temp2 = (long)(2 * cFemurLength) * c2DEC * (unsigned long)IKSW2;
  T3 = Temp1 / (Temp2 / c4DEC);
  IKA24 = GetArcCos (T3 );

  //IKFemurAngle
  FemurAngle1[LegIKLegNr] = -(long)(IKA14 + IKA24) * 180 / 3141 + 900;
  
   
}


//--------------------------------------------------------------------
//[CHECK ANGLES] Checks the mechanical limits of the servos
//--------------------------------------------------------------------
void CheckAngles(void)
{

  for (LegIndex = 0; LegIndex <= 3; LegIndex++)
  {
    CoxaAngle1[LegIndex]  = min(max(CoxaAngle1[LegIndex], (short)pgm_read_word(&cCoxaMin1[LegIndex])), (short)pgm_read_word(&cCoxaMax1[LegIndex]));
    FemurAngle1[LegIndex] = min(max(FemurAngle1[LegIndex], (short)pgm_read_word(&cFemurMin1[LegIndex])), (short)pgm_read_word(&cFemurMax1[LegIndex]));
  }

   
}

//------------------------------------------------------------------------------------------
//[OutputServoInfoForLeg] Do the output to the SSC-32 for the servos associated with
//         the Leg number passed in.
//------------------------------------------------------------------------------------------
void OutputServoInfoForLeg(byte LegIndex, short sCoxaAngle1, short sFemurAngle1)
{
  word    wCoxaSSCV;
  word    wFemurSSCV;        // cRR     0, cRF     1,cLR     2,cLF     3

  //Update Right Legs
 switch (SelectMode) {
      case 0: 

      
if (LegIndex < 2) {
    if (LegIndex == cRF && TravelLengthZ < -cTravelDeadZone){//0) {
      wCoxaSSCV = ((long)(sCoxaAngle1 + 900)) * 1000 / cPwmDiv + cPFConst + (short)pgm_read_word(&cCoxaOffset[LegIndex]);
      wFemurSSCV = ((long)(sFemurAngle1 + 900)) * 1000 / cPwmDiv + cPFConst + (short)pgm_read_word(&cFemurOffset[LegIndex]);
    } else{

      wCoxaSSCV = ((long)(-sCoxaAngle1 + 900)) * 1000 / cPwmDiv + cPFConst + (short)pgm_read_word(&cCoxaOffset[LegIndex]);
    wFemurSSCV = ((long)(-sFemurAngle1 + 900)) * 1000 / cPwmDiv + cPFConst+ (short)pgm_read_word(&cFemurOffset[LegIndex]); //original
  }
} else {// LEFT quand en face OKKKKKKKKKKKKKKK
  if (LegIndex == cLF && TravelLengthZ < -cTravelDeadZone){//SelectMode testé

    wCoxaSSCV = ((long)(-sCoxaAngle1 + 900)) * 1000 / cPwmDiv + cPFConst + (short)pgm_read_word(&cCoxaOffset[LegIndex]);
    wFemurSSCV = ((long)(-sFemurAngle1 + 900)) * 1000 / cPwmDiv + cPFConst + (short)pgm_read_word(&cFemurOffset[LegIndex]);
  } else if (LegIndex == cLR && TravelLengthZ >cTravelDeadZone){// 0 ){
    wCoxaSSCV = ((long)(-sCoxaAngle1 + 900)) * 1000 / cPwmDiv + cPFConst + (short)pgm_read_word(&cCoxaOffset[LegIndex]);
    wFemurSSCV = ((long)((long)(sFemurAngle1 + 900)) * 1000 / cPwmDiv + cPFConst + (short)pgm_read_word(&cFemurOffset[LegIndex]));
    
    }else{

  
    wCoxaSSCV = ((long)(sCoxaAngle1 + 900)) * 1000 / cPwmDiv + cPFConst + (short)pgm_read_word(&cCoxaOffset[LegIndex]);
    wFemurSSCV = ((long)((long)(sFemurAngle1 + 900)) * 1000 / cPwmDiv + cPFConst + (short)pgm_read_word(&cFemurOffset[LegIndex])); //original
  }

}

  break;
  
  case 1:
    wCoxaSSCV = ((long)(sCoxaAngle1 + 900)) * 1000 / cPwmDiv + cPFConst + (short)pgm_read_word(&cCoxaOffset[LegIndex]);
    wFemurSSCV = ((long)((long)(sFemurAngle1 + 900)) * 1000 / cPwmDiv + cPFConst + (short)pgm_read_word(&cFemurOffset[LegIndex]));
  break;
  
  default:
    // if nothing else matches, do the default
    // default is optional
    break;
  }



SSCSerial.print("#");
SSCSerial.print(pgm_read_byte(&cCoxaPin[LegIndex]), DEC);
SSCSerial.print("P");
SSCSerial.print(wCoxaSSCV, DEC);
SSCSerial.print("#");
SSCSerial.print(pgm_read_byte(&cFemurPin[LegIndex]), DEC);
SSCSerial.print("P");
SSCSerial.print(wFemurSSCV, DEC);

}




//--------------------------------------------------------------------
//[ServoDriverCommit] Updates the positions of the servos - This outputs
//         as much of the command as we can without committing it.  This
//         allows us to once the previous update was completed to quickly
//        get the next command to start
//--------------------------------------------------------------------
void ServoDriverCommit()
{
  SSCSerial.print("T");
  SSCSerial.print(SSCTime, DEC);
  SSCSerial.print("\r\n");

  PrevSSCTime = SSCTime;
}

//--------------------------------------------------------------------
// SmoothControl (From Zenta) -  This function makes the body 
//            rotation and translation much smoother while walking
//--------------------------------------------------------------------
short SmoothControl (short CtrlMoveInp, short CtrlMoveOut, byte CtrlDivider)
{
    if (fWalking)
    {
        if (CtrlMoveOut < (CtrlMoveInp - 4))
              return CtrlMoveOut + abs((CtrlMoveOut - CtrlMoveInp)/CtrlDivider);
        else if (CtrlMoveOut > (CtrlMoveInp + 4))
              return CtrlMoveOut - abs((CtrlMoveOut - CtrlMoveInp)/CtrlDivider);
    }

    return CtrlMoveInp;
}




//--------------------------------------------------------------------
//[CHECK VOLTAGE]
//Reads the input voltage and shuts down the bot when the power drops
bool CheckVoltage() {
  unsigned long ulSendTime = millis();
  //398 pour ~5.47

  g_fLowVoltageShutdown = 0;   
  
  if (!g_fLowVoltageShutdown) {
    g_RawAnalog+=analogRead(A7);
    iRaw = iRaw + 1 & 0x7;
    if (iRaw==0){
        Voltage=(word)((g_RawAnalog/8)* (float)((5.0 / 1024.)*400));
        g_RawAnalog=0;
    }
    
    // Voltage = (word)(analogRead(A7)*(5.0 / 1024.)*380);
            
          if ((ulSendTime - ulLastSend) > 2000) {
            if (Voltage <710 ){ //680   A reprogramer!!!!!!!!!!!!!!!!!!!!!!!!!
                iCntLow++;
                if (iCntLow==3){
                  XBeeOutputString("Low Batt");
                  iCntLow=0;
            }else{
               XBeeOutputVal(Voltage);
              }
            }else{
                XBeeOutputVal(Voltage);
                iCntLow=0;
                //SSCSerial.println(Voltage);
                }
            ulLastSend = ulSendTime;
          }

          
          
  }


    
  return g_fLowVoltageShutdown;
}



//==============================================================================
// [UnMap] - Converti 0->-90 128->0 255->90 
//==============================================================================
short UnMap(byte bPin)
{
    return map((short)bPin, 0, 255, -90, 90);
}

//--------------------------------------------------------------------
//[WriteOutputs] Updates the state of the leds
//--------------------------------------------------------------------
void WriteOutputs(byte Index, int fadeV)
{
        analogWrite((byte)pgm_read_byte(&cEyesPin[Index]), fadeV);
        
}

void Led(int val)
{
  for (byte i=0; i<2;i++){
    WriteOutputs(i,val);
    }
}

/*
//=============================================================================
// EEPROM stuff
//=============================================================================

//==============================================================================
// [ReadPointer] - First Read : extrait les pointeurs
//==============================================================================

void ReadPointer() { //OK
  byte Pin;
  uint8_t iSeq;
  uint8_t STime;
  uint8_t btStep;

  Addr=0;
  
  iSeq =  EEPROM.read(Addr);
  Addr++;

  for (int j = 1; j <= iSeq; j++) {
    
    btStep = EEPROM.read(Addr);
    Addr++;

    STime = EEPROM.read(Addr);
    Addr++;

    Ptr[j - 1] = EEPReadInt(Addr);
    Addr += 2;
    
    for (int i = 1; i < btStep + 1; i++) {
      for (int k = 0; k < 10; k++) {
        Pin = EEPROM.read(Addr);
        Addr ++;
      }
    }


  }
}


//==============================================================================
// [PlayStep] - PlayStep from EEPROM
//==============================================================================
void PlayStep() {
  for (byte k = 0; k < 10; k++) {
    abPin[k] = EEPROM.read(Addr);
    Addr ++;
  }
  for (int i=0;i<4;i++){
     CoxaAngle1[i]=10*UnMap(abPin[i*2]);
     FemurAngle1[i]=10*UnMap(abPin[i*2+1]);
    }
    HeadXAngle1=UnMap(abPin[8]);
    HeadYAngle1=UnMap(abPin[9]);
    
    
  StartUpdateServos();
  ServoDriverCommit();
  delay(SSCTime);
  currentStep++;
}





//==============================================================================
// [readSeq] - Read header from EEPROM
//==============================================================================

void readSeq(byte n) {
  uint8_t iStep;
  int P;
  
  Addr = Ptr[n-1];  // n-1
  
  iStep = EEPROM.read(Addr);
  Addr += sizeof(iStep);
  
  SSCTime = EEPROM.read(Addr);
  Addr ++;
  
  P = EEPReadInt(Addr);
  Addr += sizeof(P);

  SSCTime=map((byte)SSCTime,0,255,200,1000);
}


//==============================================================================
// [EEPReadInt] - Read a 2 byte integer
//     from the eeprom at the specified address
//==============================================================================
unsigned int EEPReadInt(int p_address) {

  byte lowByte = EEPROM.read(p_address);
  byte highByte = EEPROM.read(p_address + 1);

  return ((lowByte << 0) & 0xFF) + ((highByte << 8) & 0xFF00);
}




*/



//=============================================================================
// Controller stuff
//=============================================================================


//==============================================================================
// This is The function that is called by the Main program to initialize
//   the input controller.
//==============================================================================
void InitController(void)
{

  // Lets try to initialize the XBEE to use
  Serial.begin(115200);    // BUGBUG??? need to see if this baud will work here.

  // for Xbee with no flow control, maybe nothing to do here yet...
  g_diystate.fTransReadyRecvd = false;
  g_diystate.fPacketValid = false;
  g_diystate.bPacketNum = 0;


  // Clear any stuff left in the buffer
  delay(20);
  ClearXBeeInputBuffer();


  bXBeeControlMode = WALKMODE;
}





//==============================================================================
// This is code the checks for and processes input from the DIY XBee receiver
//   work
//==============================================================================
void XBeeControlInput(void)
{
  byte iNumButton;

  
  // Then try to receive a packet of information from the XBEE.
  // It will return true if it has a valid packet
  if (ReceiveXBeePacket(&g_diyp)) {
    
    if (memcmp((void*)&g_diyp, (void*)&g_diypPrev, sizeof(g_diyp)) != 0) {
      
    }

    if ( (g_diyp.s.bRJoyLR != g_diypPrev.s.bRJoyLR) || (g_diyp.s.bLJoyLR != g_diypPrev.s.bLJoyLR) || (g_diyp.s.bRJoyUD != g_diypPrev.s.bRJoyUD) ||
        (g_diyp.s.bLJoyUD != g_diypPrev.s.bLJoyUD) || (g_diyp.s.bLPot != g_diypPrev.s.bLPot) ||  (g_diyp.s.bRPot != g_diypPrev.s.bRPot) || (g_diyp.s.wButtons != g_diypPrev.s.wButtons)  )
             IddleTime=millis();

             
    // OK lets try "0" button for Start.
    if ((g_diyp.s.wButtons & (1 << 0)) && ((g_diypPrev.s.wButtons & (1 << 0)) == 0)) { //Start Button (0 on keypad) test
      if (g_fPuppyIKOn)  {
        //Turn off
        BodyPosX = 0;
        BodyPosY = -44;
        BodyPosZ = 0;
        TravelLengthX = 0;
        TravelLengthZ = 0;
        SSCTime = 600;
        XBeeOutputString("Waff");
        g_fPuppyIKOn = 0;
        
        // Led OFF (both) = 0
        Led(0);
        
       
      } else  {
        //Turn on
        SSCTime = 200;
        g_fPuppyIKOn = 1;
        XBeeOutputString("Waff Waff");
        
        // Led ON (both) = 255
        Led(255);
        delay(500);
        Led(128);
        
       
          JawAngle1=UnMap(128-50);
          MP3Serial.playFileIndex(2);
       
        
        

        
      }
    }

    if (g_fPuppyIKOn) {
      if ((g_diyp.s.wButtons & (1 << 0xa)) && ((g_diypPrev.s.wButtons & (1 << 0xa)) == 0)) { // A button - WALKMODE
        bXBeeControlMode = WALKMODE;
        SelectMode = 0;
        XBeeOutputString("Walk");
        
      }

      if ((g_diyp.s.wButtons & (1 << 0xb)) && ((g_diypPrev.s.wButtons & (1 << 0xb)) == 0)) { // B button - TRANSLATEMODE
        bXBeeControlMode = TRANSLATEMODE;
        SelectMode = 0;
        XBeeOutputString("Translate");
        
      }

      if ((g_diyp.s.wButtons & (1 << 0xd)) && ((g_diypPrev.s.wButtons & (1 << 0xd)) == 0)) { // D button - Sound ON/OFF
          if (SoundOn) {
              SoundOn=0;
              XBeeOutputString("Sound OFF");
      }else if (SoundOn==0) {
              SoundOn=1;
              XBeeOutputString("Sound ON");
          }
      }


            
      //Hack there are several places that use the 1-N buttons to select a number as an index
      // so lets convert our bitmap of which key may be pressed to a number...
      // BUGBUG:: There is probably a cleaner way to convert... Will extract buttons 1-9
      iNumButton = 0;  // assume no button
      if ((g_diyp.s.wButtons & 0x3fe) && ((g_diypPrev.s.wButtons & 0x3fe) == 0)) { // buttons 1-9
        word w = g_diyp.s.wButtons & 0x3fe;

        while ((w & 0x1) == 0)     {
          w >>= 1;
          iNumButton++;
        }
      }

       //if ((bXBeeControlMode==WALKMODE) && (GaitType == 0) && iNumButton  && (iNumButton <= 9)) { //1-9 Button Gait select   
       if (iNumButton  && (iNumButton <= 9)) { //1-9 Button Gait select   
                if ( abs(TravelLengthX)<cTravelDeadZone &&  abs(TravelLengthZ)<cTravelDeadZone )  {
                    //Switch Seq type
                    SelectMode = 1;
                    bXBeeControlMode = SEQMODE;
                    //XBeeOutputString((char*)s_asGateNames[GaitType]);
                    XBeeOutputString("Seq On");
                }
       }

      //Body Height
      BodyPosY = -44 + 0;//  g_diyp.s.bLPot / 2; 

      //Leg lift height - Right slider has value 0-255 translate to 30-93
      LegLiftHeight = 30+92/5; 

      //Walk mode
      if (bXBeeControlMode == WALKMODE) {   //RPOT =228
        TravelLengthX = -(g_diyp.s.bRJoyLR - 128)*2/3;
        TravelLengthZ = -(g_diyp.s.bRJoyUD - 128)*2/3 ;                                      
        
        HeadXAngle1 = (g_diyp.s.bLJoyLR - 128)*2;
        HeadYAngle1 =-(g_diyp.s.bLJoyUD - 128);

        #ifdef jaw
          JawAngle1=UnMap(128-(g_diyp.s.bLPot));
        #endif
        
        if (abs(TravelLengthZ)<cTravelDeadZone){
          
          GaitType=0;
          GaitSelect();
        }
        
        if  (TravelLengthX>cTravelDeadZone){//gauche
              GaitType=1;
              GaitSelect();
              TravelLengthZ=-max(abs(TravelLengthX),13);
              TravelLengthX=0;
              BodyPosY=-34;
              HeadXAngle1 = (g_diyp.s.bRJoyLR - 128)*2;
              HeadYAngle1 =-(g_diyp.s.bRJoyUD - 128) ;
        }
        
        if (TravelLengthX<-cTravelDeadZone){
              GaitType =2;
              GaitSelect();
              TravelLengthZ=-max(abs(TravelLengthX),13);
              TravelLengthX=0;
              BodyPosY=-34;
              
              HeadXAngle1 = (g_diyp.s.bRJoyLR - 128)*2;
              HeadYAngle1 =-(g_diyp.s.bRJoyUD - 128) ;

        }
        
        if (GaitType == 0 && TravelLengthZ>cTravelDeadZone){
          BPosZ=-20;
        }

          
        
      }

      //Body move
      
      if (bXBeeControlMode == TRANSLATEMODE)  {
        BodyPosX = (g_diyp.s.bRJoyLR - 128); /// 2;
        if (g_diyp.s.bRJoyUD - 128<0)
              BodyPosZ = (g_diyp.s.bRJoyUD - 128) * 2;
              else
              BodyPosZ = (g_diyp.s.bRJoyUD - 128) / 2;
      
        HeadXAngle1 = (g_diyp.s.bRJoyLR - 128)/2;
        HeadYAngle1 = (g_diyp.s.bRJoyUD - 128)/2;

        #ifdef jaw
          JawAngle1=UnMap(128-(g_diyp.s.bLPot));
        #endif
                
      }

      if (g_diyp.s.bLPot==0) 
              bPlay=0;  

      //Calculate walking time delay with Speed (128 -(g_diyp.bLSlider)/2)
      InputTimeDelay = 128 -  max( max( abs(g_diyp.s.bRJoyLR - 128),  abs(g_diyp.s.bRJoyUD - 128)),  abs(g_diyp.s.bLJoyLR - 128)) + (128 - (g_diyp.s.bRPot) / 2) ;
    }

    g_diypPrev = g_diyp; // remember the last packet

    


    
  }  else  {
    // Not a valid packet - we should go to a turned off state as to not walk into things!
    if (g_fPuppyIKOn && (g_diystate.fPacketForced ))  {
      //Turn off
      BodyPosX = 0;
      BodyPosY = -44;
      BodyPosZ = 0;
      TravelLengthX = 0;
      TravelLengthZ = 0;
      //TravelRotationY = 0;
      g_fPuppyIKOn = 0;



    }
  }

}

//=============================================================================
// Xbee stuff
//=============================================================================

//=============================================================================
// void WriteToXBee - output a buffer to the XBEE.  This is used to abstract
//            away the actual function needed to talk to the XBEE
//=============================================================================
inline void WriteToXBee(byte *pb, byte cb) __attribute__((always_inline));
void WriteToXBee(byte *pb, byte cb)
{
  Serial.write((byte*)pb, cb);
}


//==============================================================================
// [SendXBeePacket] - Simple helper function to send the 4 byte packet header
//     plus the extra data if any
//      gosub SendXBeePacket[bPacketType, cbExtra, pExtra]
//==============================================================================
void SendXBeePacket(byte bPHType, byte cbExtra, byte *pbExtra)
{
  // Tell system to now output to the xbee
  byte abPH[9];
  byte *pbT;
  byte bChkSum;
  int i;

  // We need to setup the xbee Packet
  abPH[0] = 0x7e;                      // Command prefix
  abPH[1] = 0;                          // msb of size
  abPH[2] = cbExtra + XBEE_API_PH_SIZE + 5; // size LSB
  abPH[3] = 0x81;                           // Send to 16 bit address.

  g_diystate.bPacketNum = g_diystate.bPacketNum + 1;
  if (g_diystate.bPacketNum == 0)
    g_diystate.bPacketNum = 1;        // Don't pass 1 as this says no ack
  abPH[4] = g_diystate.bPacketNum;      // frame number
  abPH[5] = 0;      // Our current destination MSB/LSB
  abPH[6] = 0;
  abPH[7] = 0;                          // No Options

  abPH[8] = bPHType;

  // Now compute the initial part of the checksum
  bChkSum = 0;
  for (i = 3; i <= 8; i++)
    bChkSum += abPH[i];

  // loop through the extra bytes in the exta to build the checksum;
  pbT = pbExtra;
  for (i = 0; i < cbExtra; i++)
    bChkSum += *pbT++;                // add each byte to the checksum

  // Ok lets output the fixed part
  WriteToXBee(abPH, 9);

  // Ok lets write the extra bytes if any to the xbee
  if (cbExtra)
    WriteToXBee(pbExtra, cbExtra);

  // Last write out the checksum
  bChkSum = 0xff - bChkSum;
  WriteToXBee(&bChkSum, 1);

}

//==============================================================================
// [XBeeOutputVal] - Simple wrapper function to pass a word value back to
//            remote control to display
//==============================================================================
void XBeeOutputVal(word w)
{
  SendXBeePacket(XBEE_RECV_DISP_VAL, sizeof(w), (byte*)&w);
}

//==============================================================================
// [XBeeOutputVal0] - Simple wrapper function to pass a word value back to
//            remote control to display
//==============================================================================
void XBeeOutputVal0(word w)
{
  SendXBeePacket(XBEE_RECV_DISP_VAL0, sizeof(w), (byte*)&w);
}


//////////////////////////////////////////////////////////////////////////////
//==============================================================================
// [APIRecvPacket - try to receive a packet from the XBee.
//        - Will return TRUE if it receives something, else false
//        - Pass in buffer to receive packet.  Assumed it is big enough...
//        - pass in timeout if zero will return if no data...
//
//==============================================================================
byte APIRecvPacket(ulong Timeout)
{
  byte cbRead;
  byte abT[3];
  byte bChksum;
  int i;

  short wPacketLen;
  if (Timeout == 0)
  {
    if (!Serial.available())
      return 0;        // nothing waiting for us...
    Timeout = 100;    // .1 second?

  }

  Serial.setTimeout(Timeout);


  // Now lets try to read in the data from the xbee...
  // first read in the delimter and packet length
  // We now do this in two steps.  The first to try to resync if the first character
  // is not the proper delimiter...

  do {
    cbRead = Serial.readBytes(abT, 1);
    if (cbRead == 0)
      return 0;
    //else{SSerial.print(abT[0]);}
  }
  while (abT[0] != 0x7e);//0x7e



  cbRead = Serial.readBytes(abT, 2);
  if (cbRead != 2)
    return 0;                // did not read in full header or the header was not correct.

  wPacketLen = (abT[0] << 8) + abT[1];

  if (wPacketLen >= sizeof(g_diystate.bAPIPacket)) {

    ClearXBeeInputBuffer();  // maybe clear everything else out.
    return 0;  // Packet bigger than expected maybe bytes lost...
  }

  // Now lets try to read in the packet
  cbRead = Serial.readBytes(g_diystate.bAPIPacket, wPacketLen + 1);


  // Now lets verify the checksum.
  bChksum = 0;
  for (i = 0; i < wPacketLen; i++)
    bChksum = bChksum + g_diystate.bAPIPacket[i];             // Add that byte to the buffer...


  if (g_diystate.bAPIPacket[wPacketLen] != (0xff - bChksum)) {

    ClearXBeeInputBuffer();  // maybe clear everything else out.
    return 0;                // checksum was off
  }

  return wPacketLen;    // return the packet length as the caller may need to know this...
}




/////////////////////////////////////////////////////////////////////////////


//==============================================================================
// [ClearXBeeInputBuffer] - This simple helper function will clear out the input
//                        buffer from the XBEE
//==============================================================================
void ClearXBeeInputBuffer(void)
{

  Serial.flush();    // clear out anything that was queued up...
  while (Serial.read() != -1)
    ;    // just loop as long as we receive something...

}


//==============================================================================
// [ReceiveXBeePacket] - This function will try to receive a packet of information
//         from the remote control over XBee.
//
// the data in a standard packet is arranged in the following byte order:
//    0 - Buttons High
//    1 - Buttons Low
//     2 - Right Joystick L/R
//    3 - Right Joystick U/D
//    4 - Left Joystick L/R
//    5 - Left Joystick U/D
//     6 - Right Slider
//    7 - Left Slider
//==============================================================================
boolean ReceiveXBeePacket(PDIYPACKET pdiyp)
{
  uint8_t cbRead;
  uint8_t bDataOffset;
  ulong ulCurrentTime;
  ulong ulTimeDiffMS;
  boolean _fPacketValidPrev = g_diystate.fPacketValid;        // Save away the previous state as this is the state if no new data...
  boolean _fNewPacketAvail = false;

  g_diystate.fPacketForced = false;
  g_diystate.fPacketValid = false;
  g_diystate.fNewPacket = false;      // Did we receive a new packet?

  //    We will first see if we have a packet header waiting for us.
  //  BUGBUG:: Question should I loop after I process a package and only get out of the
  //             loop when I have no more data, or only process one possible message?
  //            Maybe depends on message?
  for (;;)
  {
    if (!Serial.available())
      break;        // no input available, break from this loop

    // The XBEE has sent us some data so try to get a packet header
    cbRead = APIRecvPacket(10);        // Lets read in a complete packet.

    if (!cbRead) {
      break;                            // Again nothing read?
    }



    if (g_diystate.bAPIPacket[0] == XBEE_RECV_REQ_NEW) //0x81)
    {
      bDataOffset = 5;   // Received packet with 16 bit addressing
    }
    else if (g_diystate.bAPIPacket[0] == XBEE_RECV_REQ_DATA)
      bDataOffset = 11;                // Received packet with 64 bit addressing
    else if (g_diystate.bAPIPacket[0] == 0x89) {
      if (g_diystate.bAPIPacket[2] != g_diystate.bTxStatusLast) {
        g_diystate.bTxStatusLast = g_diystate.bAPIPacket[2];

      }
      continue;
    }
    else
      break;                            // Invalid packet lets bail from this loop.

    // Change CB into the number of extra bytes...
    cbRead -= (bDataOffset + 1);        // Ph is only 1 byte long now...

    //-----------------------------------------------------------------------------
    // [XBEE_TRANS_DATA]
    //-----------------------------------------------------------------------------
    if (g_diystate.bAPIPacket[bDataOffset + 0] == XBEE_TRANS_DATA) {
      if (cbRead >= sizeof(DIYPACKETORIG)) {
        pdiyp->s.wButtons = g_diystate.bAPIPacket[bDataOffset + 1] + (g_diystate.bAPIPacket[bDataOffset + 2] << 8);

        // Simple copy the memory down
        g_diystate.cbPacketSize = (cbRead <= sizeof(DIYPACKET)) ? cbRead : sizeof(DIYPACKET);

        memcpy(&pdiyp->ab[2], &g_diystate.bAPIPacket[bDataOffset + 3], g_diystate.cbPacketSize);

        // process first as higher number of these come in...
        g_diystate.fPacketValid = true;    // data is valid
        g_diystate.fPacketForced = g_diystate.fReqDataForced;    // Was the last request forced???
        g_diystate.fNewPacket = true;      // Did we receive a new packet?
        g_diystate.fDataPacketsReceived = true;
        // BUGBUG:: Assuming 16 bit addressing
        g_diystate.ulLastPacket = millis();
        return true;    //          // get out quick!
      }

    }


    //-----------------------------------------------------------------------------
    // [XBEE_TRANS_SEQ_DATA]
    //-----------------------------------------------------------------------------
    if (g_diystate.bAPIPacket[bDataOffset + 0] == XBEE_TRANS_SEQ_DATA) {
    
        memcpy(&abPin, &g_diystate.bAPIPacket[bDataOffset + 1], sizeof(abPin));

     

     for (int i=0;i<4;i++){
      CoxaAngle1[i]=10*UnMap(abPin[i*2]);
      FemurAngle1[i]=10*UnMap(abPin[i*2+1]);
      
      if ((prev_CoxaAngle[i] != CoxaAngle1[i]) || (prev_FemurAngle[i] != FemurAngle1[i]) ) 
        IddleTime=millis();

      prev_CoxaAngle[i] = CoxaAngle1[i];
      prev_FemurAngle[i] = FemurAngle1[i];
  
    }
    HeadXAngle1=UnMap(abPin[8]);
    HeadYAngle1=UnMap(abPin[9]);
    SSCTime=map(abPin[10],0,255,200,1000);
    JawAngle1=UnMap(128-abPin[11]);
    
    if ((prev_HeadXAngle != HeadXAngle1) || (prev_HeadYAngle != HeadYAngle1) ) 
        IddleTime=millis();

    prev_HeadXAngle = HeadXAngle1;
    prev_HeadYAngle = HeadYAngle1; 
    
    SeqOk=true;
    
        // process first as higher number of these come in...
        g_diystate.fPacketValid = true;    // data is valid
        g_diystate.fPacketForced = g_diystate.fReqDataForced;    // Was the last request forced???
        g_diystate.fNewPacket = true;      // Did we receive a new packet?
        g_diystate.fDataPacketsReceived = true;
        // BUGBUG:: Assuming 16 bit addressing
        g_diystate.ulLastPacket = millis();
        return true;    //          // get out quick!
  }

  }

  //-----------------------------------------------------------------------------
  // Exited above loop now See if we we are in some form of timeout condition.
  // We should receive a new data message about 30 times per second.  If we
  // have gone for a reasonably long time without receiving a message, assume
  // transmitter went away!
  //-----------------------------------------------------------------------------
  // Only send when we know the transmitter is ready.  Also if we are in the New data only mode don't ask for data unless we have been
  //    old there
  // is new data. We relax this a little and be sure to ask for data every so often as to make sure the remote is still working...
  //
  if (g_diystate.fDataPacketsReceived) {
    ulCurrentTime = millis();
    ulTimeDiffMS = ulCurrentTime - g_diystate.ulLastPacket;

    // See if we exceeded a global timeout.  If so let caller know so they can stop themself if necessary...
    if (ulTimeDiffMS > 250) {
      g_diystate.fPacketValid = 0;
      return false;
    }

    g_diystate.fPacketValid = _fPacketValidPrev;    // Say the data is in the same state as the previous call...
  }

  return g_diystate.fPacketValid;
}
